#!/bin/sh
exec ruby19 -x "$0" "$@"
#!ruby
# coding: utf-8
# vi:set ts=3 sw=3:
# vim:set sts=0 noet:

=begin
Copyright (c) 2010 KISHIMOTO, Makoto

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
=end

require_relative "jtokenizer"

class Tango
	attr_accessor :surf, :left_id, :right_id, :cost

	def initialize surf, left_id, right_id, cost
		@surf, @left_id, @right_id, @cost = surf, left_id, right_id, cost
	end
end

class TrieNode
	attr_accessor :tango
	attr_reader :skipstr
	attr_accessor :children

	def initialize
		@tango = nil
		@skipstr = ""
		@children = {}
	end
end

def compress_trie trie_node
	if (trie_node.children.size == 1) and not trie_node.tango then
		while (trie_node.children.size == 1) and not trie_node.tango do
			trie_node.children.each {|key, node|
				trie_node.skipstr << key
				trie_node.tango = node.tango
				trie_node.children = node.children
			}
		end
		compress_trie trie_node
	else
		trie_node.children.each_value {|node|
			compress_trie node
		}
	end
end

def make_trie jdic
	tango_list = []
	reminder = {}
	jdic.each_line {|ln|
		ln.chomp!
		lst = ln.split(",")
		if lst.size > 15 then  # exception of ","
			next
		end
		lst.slice!(4 .. -1)
		s = JTokenizer.zenkaku2hankaku lst[0]
		s = s.tr "A-Z", "a-z"
		s = s.gsub(/[\s]/){""}
		if s.empty? then
			next
		end
		if reminder.key? s then
			next
		end
		tango = Tango.new s, lst[1].to_i, lst[2].to_i, lst[3].to_i
		tango_list << tango
		reminder[s] = true
	}

	tango = Tango.new ",", 10, 10, 617
	tango_list << tango

	trie_root = TrieNode.new

	tango_list.each {|tango|
		trie_node = trie_root
		tango.surf.each_char {|char|
			unless trie_node.children.key? char then
				trie_node.children[char] = TrieNode.new
			end
			trie_node = trie_node.children[char]
		}
		trie_node.tango = tango
	}

	compress_trie trie_root

	trie_root
end

dir = File.dirname __FILE__

trie = nil
(lambda {
	jdic = nil
	open("#{dir}/naist-jdic.csv", "r:EUC-JP:UTF-8") {|file|
		jdic = file.read
	}
	trie = make_trie jdic
}).call

db = JTokenizer::TrieDB.open
begin
	JTokenizer.build_db db, trie
ensure
	db.close
end
